import xbmc
import xbmcgui
from database import dbio
from dialogs import context
from emby import listitem
from core import common
from . import utils, pluginmenu, playerops

SelectOptions = {'Refresh': utils.Translate(30410), 'Delete': utils.Translate(30409), 'Addon': utils.Translate(30408), 'AddFav': utils.Translate(30405), 'RemoveFav': utils.Translate(30406), 'SpecialFeatures': utils.Translate(33231), "Watch together": utils.Translate(33495)}


def load_item():
    item = None
    ServerId = xbmc.getInfoLabel('ListItem.Property(embyserverid)')
    emby_id = xbmc.getInfoLabel('ListItem.Property(embyid)')

    if not ServerId:
        for ServerId in utils.EmbyServers:
            kodi_id = xbmc.getInfoLabel('ListItem.DBID')
            media = xbmc.getInfoLabel('ListItem.DBTYPE')
            embydb = dbio.DBOpenRO(ServerId, "load_item")
            item = embydb.get_item_by_KodiId_KodiType(kodi_id, media)
            dbio.DBCloseRO(ServerId, "load_item")

            if item:
                item = item[0]
                break

        return item, ServerId

    pluginmenu.reset_querycache() # Clear Cache
    return (emby_id,), ServerId

def delete_item(LoadItem, item=None, ServerId=""):  # threaded by caller
    if utils.Dialog.yesno(heading=utils.addon_name, message=utils.Translate(33015)):
        if LoadItem:
            item, ServerId = load_item()

        if item:
            utils.EmbyServers[ServerId].API.delete_item(item[0])
            utils.EmbyServers[ServerId].library.removed([item[0]])
            xbmc.executebuiltin("Container.Refresh()")

def specials():
    SpecialFeaturesSelections = []
    item, ServerId = load_item()

    if not item:
        return

    # Load SpecialFeatures
    embydb = dbio.DBOpenRO(ServerId, "specials")
    SpecialFeaturesIds = embydb.get_special_features(item[0])

    for SpecialFeaturesId in SpecialFeaturesIds:
        SpecialFeaturesMediasources = embydb.get_mediasource(SpecialFeaturesId[0])
        SpecialFeaturesSelections.append({"Name": SpecialFeaturesMediasources[0][4], "Id": SpecialFeaturesId[0]})

    dbio.DBCloseRO(ServerId, "specials")
    MenuData = []

    for SpecialFeaturesSelection in SpecialFeaturesSelections:
        MenuData.append(SpecialFeaturesSelection['Name'])

    resp = utils.Dialog.select(utils.Translate(33231), MenuData)

    if resp < 0:
        return

    ItemData = SpecialFeaturesSelections[resp]
    SpecialFeatureItem = utils.EmbyServers[ServerId].API.get_Item(ItemData['Id'], ['Movie'], True, False, True, True)

    if SpecialFeatureItem:
        li = listitem.set_ListItem(SpecialFeatureItem, ServerId)
        path, _ = common.get_path_type_from_item(ServerId, SpecialFeatureItem)
        li.setProperty('path', path)
        Pos = playerops.GetPlayerPosition(1) + 1
        utils.Playlists[1].add(path, li, index=Pos)
        playerops.PlayPlaylistItem(1, Pos)

def select_menu(ListItemLabel):
    options = []
    item, ServerId = load_item()

    if not item:
        return

    if len(item) > 9:
        if item[10]:
            options.append(SelectOptions['RemoveFav'])
        else:
            options.append(SelectOptions['AddFav'])

    options.append(SelectOptions['Refresh'])

    if utils.enableContextDelete:
        options.append(SelectOptions['Delete'])

    if item[2] in ('Episode', 'Movie', 'MusicVideo', 'Audio'):
        options.append(SelectOptions['Watch together'])

    options.append(SelectOptions['Addon'])
    context_menu = context.ContextMenu("script-emby-context.xml", *utils.CustomDialogParameters)
    context_menu.PassVar(options)
    context_menu.doModal()

    if context_menu.is_selected():
        selected_option = context_menu.get_selected()

        if selected_option == SelectOptions['Refresh']:
            utils.EmbyServers[ServerId].API.refresh_item(item[0])
        elif selected_option == SelectOptions['AddFav']:
            utils.EmbyServers[ServerId].API.favorite(item[0], True)
        elif selected_option == SelectOptions['RemoveFav']:
            utils.EmbyServers[ServerId].API.favorite(item[0], False)
        elif selected_option == SelectOptions['Addon']:
            xbmc.executebuiltin(f"Addon.OpenSettings({utils.PluginId})")
        elif selected_option == SelectOptions['Delete']:
            delete_item(False, item, ServerId)
        elif selected_option == SelectOptions['Watch together']:
            ActiveSessions = utils.EmbyServers[ServerId].API.get_active_sessions()
            ActiveUsers = []
            ActiveUsersSessions = []

            for ActiveSession in ActiveSessions:
                if ActiveSession['SupportsRemoteControl'] and ActiveSession['Id'] != utils.EmbyServers[ServerId].EmbySession[0]['Id']:
                    ActiveUsers.append(f"{ActiveSession['DeviceName']}, {ActiveSession['UserName']}")
                    ActiveUsersSessions.append(ActiveSession['Id'])

            Selections = utils.Dialog.multiselect(utils.Translate(33494), ActiveUsers)

            if not Selections:
                return

            ProgressBar = xbmcgui.DialogProgress()
            ProgressBar.create(utils.Translate(33495))
            playerops.WatchTogetherParticipants = [utils.EmbyServers[ServerId].EmbySession[0]['Id']]
            playerops.init_WatchTogether(item[0], utils.EmbyServers[ServerId])

            for Selection in Selections:
                utils.EmbyServers[ServerId].API.send_text_msg(ActiveUsersSessions[Selection], "watch together request", f"{item[0]};{utils.EmbyServers[ServerId].EmbySession[0]['Id']};{ListItemLabel};60", True)

            # wait for clients
            WaitFactor = int(int(utils.watchtogeter_wait_clients) / 10)

            for Index in range(1, WaitFactor * 100):
                ProgressBar.update(int(Index / WaitFactor), utils.Translate(33492))

                if check_WatchTogeter(ProgressBar, ServerId, ListItemLabel):
                    return

                if len(Selections) + 1 == len(playerops.WatchTogetherParticipants):
                    break

            # Force clients to participate
            if utils.watchtogeter_force_clients:
                # Stagger commands to give clients time to catch up
                for Selection in Selections:
                    if ActiveUsersSessions[Selection] not in playerops.WatchTogetherParticipantsAcks:
                        playerops.WatchTogetherParticipants.append(ActiveUsersSessions[Selection])
                        utils.EmbyServers[ServerId].API.send_play(ActiveUsersSessions[Selection], item[0], "PlayNow", 0, True)
                        utils.EmbyServers[ServerId].API.send_pause(ActiveUsersSessions[Selection], True)

                for Selection in Selections:
                    if ActiveUsersSessions[Selection] not in playerops.WatchTogetherParticipantsAcks:
                        if utils.sleep(5):
                            return

                        utils.EmbyServers[ServerId].API.send_pause(ActiveUsersSessions[Selection], True)
                        utils.EmbyServers[ServerId].API.send_seek(ActiveUsersSessions[Selection], 0, True)
                        utils.EmbyServers[ServerId].API.send_pause(ActiveUsersSessions[Selection], True)

             # give time to prepare streams for all client devices
            ProgressBar.update(0, utils.Translate(33493))

            # Delay playback
            WaitFactor = int(int(utils.watchtogeter_start_delay) / 10)

            for Index in range(1, WaitFactor * 100):
                ProgressBar.update(int(Index / WaitFactor), utils.Translate(33493))

                if check_WatchTogeter(ProgressBar, ServerId, ListItemLabel):
                    return

            ProgressBar.close()
            playerops.Unpause()

def check_WatchTogeter(ProgressBar, ServerId, ListItemLabel):
    if utils.sleep(0.1): # wait here for user confirmation or timeout
        ProgressBar.close()
        return True

    if ProgressBar.iscanceled():
        playerops.terminate_WatchTogether()

        for SessionId in playerops.WatchTogetherParticipantsAcks:
            if SessionId != utils.EmbyServers[ServerId].EmbySession[0]['Id']:
                utils.EmbyServers[ServerId].API.send_text_msg(SessionId, "watch together cancel", ListItemLabel)

        ProgressBar.close()
        return True

    return False

def Record():
    Temp = xbmc.getInfoLabel('ListItem.EPGEventIcon') # Icon path has Emby's EPG programId assinged (workaround)
    Temp = Temp[Temp.find("@") + 1:].replace("/","")
    Temp = Temp.split("Z")
    Timers = utils.EmbyServers[Temp[0]].API.get_timer(Temp[1])
    TimerId = 0

    if Timers:
        for Timer in Timers:
            if Timer['ProgramId'] == Temp[1]:
                TimerId = Timer['ProgramInfo']['TimerId']
                break

    if TimerId: # Delete recording
        if utils.Dialog.yesno(heading=utils.addon_name, message=utils.Translate(33496)):
            utils.EmbyServers[Temp[0]].API.delete_timer(TimerId)
    else: # Add recoding
        if utils.Dialog.yesno(heading=utils.addon_name, message=utils.Translate(33497)):
            utils.EmbyServers[Temp[0]].API.set_timer(Temp[1])
